'''Automatic active(current/up-level) url highlighting(adding css class)
with django template tag
'''
from lxml.html import fromstring, tostring
from django import template
from django.core.cache import cache
try:
    from hashlib import md5 as md5_constructor
except ImportError:
    from django.utils.hashcompat import md5_constructor
from classytags.core import Tag, Options
from classytags.arguments import MultiKeywordArgument
from django_activeurl import settings


register = template.Library()


class ActiveUrl(Tag):
    '''By default additional css class is "active"
        for parent tag of all <a> which parent is <li>.
        Quick example
        {% activeurl %}
            <ul>
                <li>
                    <a href="/page/">/page/</a>
                </li>
            </ul>
        {% endactiveurl %}
        will be rendered into
            <ul>
                <li class="active">
                    <a href="/page/">page</a>
                </li>
            </ul>
        if request.get_full_path() starts with /page/
        starts with logic decided to apply "active" status
        for up-level urls from current url.
    '''

    name = 'activeurl'

    # template tag arguments
    options = Options(
        MultiKeywordArgument('kwargs', required=False),
        blocks=[('endactiveurl', 'nodelist')],
    )

    def render_tag(self, context, kwargs, nodelist):
        '''renders html with "active" urls inside template tag'''
        # set attributes from kwargs
        default_kwargs = settings.DEFAULT_KWARGS
        default_kwargs.update(kwargs)
        kwargs = default_kwargs
        css_class = kwargs['css_class']
        parent_tag = kwargs['parent_tag']

        # flag for preventing rendering, when no "active" urls found
        processed = False

        # get request from context
        request = context['request']

        # get full path from request
        full_path = request.get_full_path()

        # render content inside template tag
        context.push()
        content = nodelist.render(context)

        # try to take rendered html with "active" urls from cache
        if settings.CACHE_ACTIVE_URL:
            cache_key = settings.CACHE_ACTIVE_URL_PREFIX \
                + md5_constructor(
                    content + css_class + parent_tag + full_path
                ).hexdigest()

            from_cache = cache.get(cache_key)
            if from_cache:
                return from_cache

        # build tree from content inside template tag
        tree = fromstring(content)

        # xpath query to get all parents tags
        els = tree.xpath('//%s' % parent_tag)
        for el in els:
            # xpath query to get all <a> inside current tag
            urls = el.xpath('a')
            for url in urls:
                # check non empty href parameter
                if 'href' in url.attrib.keys():
                    # skip "root" url
                    if url.attrib['href'] != '/':
                        # compare href parameter with full path
                        if full_path.startswith(url.attrib['href']):
                            # check parent tag has "class" attribute
                            if 'class' in el.attrib.keys():
                                # prevent multiple "class" adding
                                if not css_class in el.attrib['class']:
                                    # check for empty "class" attribute
                                    if el.attrib['class']:
                                        # append "active" class
                                        el.attrib['class'] += ' ' + css_class
                                    else:
                                        # set "class" attribute
                                        el.attrib['class'] = css_class
                            else:
                                # create "class" attribute
                                el.attrib['class'] = css_class
                            # flag for rendering tree
                            processed = True
                            # stop checking other <a> inside current parent_tag
                            break

        context.pop()

        # checking status of founding "active" url
        if processed:
            # build html from tree
            content = tostring(tree)

            # write rendered html to cache, if caching is enabled
            if settings.CACHE_ACTIVE_URL:
                cache.set(cache_key, content,
                          settings.CACHE_ACTIVE_URL_TIMEOUT)

        return content

# register new template tag
register.tag(ActiveUrl)



from django.conf import settings


CACHE_ACTIVE_URL = getattr(settings, 'CACHE_ACTIVE_URL', True)
CACHE_ACTIVE_URL_TIMEOUT = getattr(
    settings, 'CACHE_ACTIVE_URL_TIMEOUT', 60 * 60 * 24)
CACHE_ACTIVE_URL_PREFIX = getattr(
    settings, 'CACHE_ACTIVE_URL_PREFIX', 'django_activeurl.')
DEFAULT_KWARGS = {'css_class': 'active', 'parent_tag': 'li'}



import codecs
from setuptools import setup


classifiers = '''\
Framework :: Django
Environment :: Web Environment
Intended Audience :: Developers
Topic :: Internet :: WWW/HTTP
License :: OSI Approved :: Apache Software License
Development Status :: 4 - Beta
Natural Language :: English
Programming Language :: Python :: 2.6
Programming Language :: Python :: 2.7
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Programming Language :: Python
Programming Language :: Python :: Implementation :: CPython
'''

description = 'Easy to use active url highlighting for django'

packages = ['django_activeurl', 'django_activeurl.templatetags', ]


def long_description():
    with codecs.open('README.rst', encoding='utf8') as f:
        rst = f.read()
    return rst

setup(
    name='django-activeurl',
    version='0.0.2',
    packages=packages,
    description=description,
    long_description=long_description(),
    author='hellysmile',
    author_email='hellysmile@gmail.com',
    url='https://github.com/hellysmile/django-activeurl/',
    zip_safe=False,
    install_requires=[
        'django >= 1.3, <1.5',
        'lxml >= 2.3.5',
        'django-classy-tags >= 0.4',
    ],
    license='http://www.apache.org/licenses/LICENSE-2.0',
    classifiers=filter(None, classifiers.split('\n')),
    keywords=[
        "django", "url", "link", "active", "css", "templatetag",
    ],
    setup_requires=['nose>=1.2.1']
)
